#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e14:p2714|e15:p2714> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=tensorflow
origpkgver=v1_3_0
pkgver=${origpkgver}b
ssibuildshims_version=v1_02_00
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//' | sed -e 's/a//'`
pkgtarname=${package}-${pkgdotver}

get_this_dir

get_ssibuildshims

source define_basics --

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

if [ "${extraqual}" = "debug" ]
then
    cflg="-g -O0"
elif [ "${extraqual}" = "prof" ]
then
    cflg="-O3 -g -DNDEBUG -fno-omit-frame-pointer"
fi

if [[ "${basequal}" == e1[45]:p2714 ]]
then
  cxxflg="${cflg} -std=c++14 -Wno-deprecated-declarations"
else
  usage
  ssi_die "Unrecognized qualifer ${basequal}"
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

if [ -z ${PROTOBUF_FQ_DIR} ]
then
   echo "ERROR: failed to setup protobuf"
   exit 1
fi
if [ -z ${PYTHONHOME} ]
then
   echo "ERROR: PYTHONHOME is undefined"
   echo "       something is very wrong"
   exit 1
fi

echo "Building ${package} in ${pkgdir}"

rm -rf ${pkgdir}
mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi

makedir=${pkgdir}/${pkgtarname}/tensorflow/contrib/makefile

allcores=`${SSIBUILDSHIMS_DIR}/bin/ncores`
(( ncore=${allcores}/2 ))
export ncore

cd ${pkgdir}
tar -xf ${tardir}/${pkgtarname}.tar || ssi_die "problem unwinding tar file"

if [ ${OS1} = "Darwin" ];then
  patchfile=${patchdir}/tensorflow-osx.patch
else
  patchfile=${patchdir}/tensorflow.patch
fi
patch -b -p1 < ${patchfile} || ssi_die "ERROR: failed to apply patch."
cp -p ${patchdir}/build_all_linux.sh ${makedir}/ || ssi_die "failed to copy build_all_linux.sh"
cp -p ${patchdir}/build_all_osx.sh ${makedir}/ || ssi_die "failed to copy build_all_linux.sh"
cp -p ${patchdir}/rename_eigen.sh ${makedir}/ || ssi_die "failed to copy rename_eigen.sh"
cp -p ${patchdir}/install_all.sh ${makedir}/ || ssi_die "failed to copy install_all.sh"

set -x

cd ${pkgdir}

if [ ${OS1} = "Darwin" ];then
  ${makedir}/build_all_osx.sh || ssi_die "build failed"
else
  ${makedir}/build_all_linux.sh || ssi_die "build failed"
fi

set +x

cd ${pkgdir}

if [ ! -d ${pkgdir}/lib ]
then
   echo "ERROR: failed to create ${pkgdir}/lib"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS} || ssi_die "ERROR: setup ${package} ${pkgver} -q ${fullqual} failed"
echo "${package} is installed at ${TENSORFLOW_FQ_DIR}"

# this must be last 
if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/lib ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

exit 0
